# Generated by Django 3.0.10 on 2020-10-19 18:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('invoices', '0001_initial'),
        ('customers', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ServiceCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Category name')),
            ],
            options={
                'verbose_name': 'Service category',
                'verbose_name_plural': 'Service categories',
            },
        ),
        migrations.CreateModel(
            name='ServiceItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, db_index=True, max_length=255, verbose_name='Service name')),
                ('code', models.CharField(blank=True, db_index=True, max_length=255, verbose_name='Code')),
                ('price', models.FloatField(verbose_name='Price')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='services.ServiceCategory', verbose_name='Category')),
            ],
            options={
                'verbose_name': 'Service item',
                'verbose_name_plural': 'Service items',
                'ordering': ('code',),
            },
        ),
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('qty', models.IntegerField(default=1, verbose_name='Quantity')),
                ('price', models.FloatField(verbose_name='Price')),
                ('customer', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='services', to='customers.Customer', verbose_name='Customer')),
                ('item', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='services.ServiceItem', verbose_name='Service item')),
                ('sale', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='services', to='invoices.Sale', verbose_name='Sale')),
            ],
            options={
                'verbose_name': 'Service',
                'verbose_name_plural': 'Service',
            },
        ),
    ]




from django.contrib.admin.views.decorators import staff_member_required
from django.http.response import JsonResponse, HttpResponseBadRequest
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

from pagination import paginate
from basement.admin import render
from invoices.models import Sale

from services.forms import ReportForm, SearchServiceForm
from services.models import ServiceCategory, ServiceItem, Service


@staff_member_required
def print_service_items(request):
    return render(request, 'services/print-service-items.html', {
        'categories': (
            ServiceCategory
                .objects
                .all()
                .prefetch_related('items')
            )
    })


@staff_member_required
def print_services(request, sale_id):

    sale = get_object_or_404(Sale, id=sale_id)

    return render(request, 'services/print-services.html', {
        'sale': sale,
        'services': sale.services.all()
    })


@staff_member_required
def get_report(request):

    form = ReportForm(request)

    services = Service.objects.filter(
        created__date__range=[
            form.cleaned_data['date_from'],
            form.cleaned_data['date_to']
        ]
    ).order_by('created')

    context = {
        'services': services,
        'form': form,
        'totals': {
            'qty': sum([s.qty for s in services]),
            'grand_total': sum([s.subtotal for s in services])
        }
    }

    context.update(form.cleaned_data)

    return render(request, 'services/report.html', context)


@staff_member_required
def get_service_items(request):

    form = SearchServiceForm(data=request.GET)

    if not form.is_valid():
        return HttpResponseBadRequest('Invalid form')

    queryset = ServiceItem.objects.search(**form.cleaned_data)

    page = paginate(request, queryset, per_page=50)

    return JsonResponse({
        'items': render_to_string('services/service-items.html', {
            'page_obj': page
        }),
        'has_next': page.has_next(),
        'next_page_url': '{}?{}'.format(
            request.path, page.next_page_number().querystring)
    })


@csrf_exempt
@require_POST
@staff_member_required
def add_service(request, sale_id):

    sale = get_object_or_404(Sale, id=sale_id)

    service_item = get_object_or_404(
        ServiceItem, id=request.POST.get('service_id'))

    try:
        service = Service.objects.get(
            sale=sale,
            item=service_item
        )
        service.qty += 1
    except Service.DoesNotExist:
        service = Service(
            customer=sale.customer,
            sale=sale,
            item=service_item,
            price=service_item.price
        )

    service.save()

    return JsonResponse({
        'status': 'OK',
        'html': service.render(),
        'item_id': service.id,
        'total': sale.serialize_totals()
    })


@csrf_exempt
@require_POST
@staff_member_required
def remove_service(request, service_id):

    service = get_object_or_404(Service, id=service_id)

    sale = service.sale

    service.delete()

    return JsonResponse({
        'message': _('Service removed'),
        'total': sale.serialize_totals()
    })




from django import template
from django.apps import apps


register = template.Library()


@register.simple_tag()
def get_service_categories():
    return apps.get_model('services', 'ServiceCategory').objects.all()




from django.db import models
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from services.managers import ServiceItemManager


class ServiceCategory(models.Model):

    name = models.CharField(_('Category name'), max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Service category')
        verbose_name_plural = _('Service categories')


class ServiceItem(models.Model):

    category = models.ForeignKey(
        ServiceCategory,
        on_delete=models.CASCADE,
        related_name='items',
        verbose_name=_('Category'))

    name = models.CharField(
        _('Service name'), max_length=255, blank=True, db_index=True)

    code = models.CharField(
        _('Code'), max_length=255, blank=True, db_index=True)

    price = models.FloatField(_('Price'))

    objects = ServiceItemManager()

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('code', )
        verbose_name = _('Service item')
        verbose_name_plural = _('Service items')


class Service(models.Model):

    sale = models.ForeignKey(
        'invoices.Sale',
        on_delete=models.PROTECT,
        related_name='services',
        verbose_name=_('Sale'),
        blank=True,
        null=True
    )

    customer = models.ForeignKey(
        'customers.Customer',
        on_delete=models.PROTECT,
        related_name='services',
        verbose_name=_('Customer'),
        blank=True,
        null=True
    )

    item = models.ForeignKey(
        ServiceItem,
        on_delete=models.PROTECT,
        verbose_name=_('Service item')
    )

    created = models.DateTimeField(_('Creation date'), auto_now_add=True)

    qty = models.IntegerField(_('Quantity'), default=1)

    price = models.FloatField(_('Price'))

    def __str__(self):
        if self.customer:
            return '{} - {}'.format(self.item.name, self.customer.name)

        return self.item.name

    def render(self):
        return render_to_string('services/item.html', {'object': self})

    @property
    def name(self):
        return self.item.name

    @property
    def code(self):
        return self.item.code

    @property
    def subtotal(self):
        return self.price * self.qty

    @property
    def remove_url(self):
        return reverse_lazy('services:remove-service', args=[self.pk])

    class Meta:
        verbose_name = _('Service')
        verbose_name_plural = _('Service')




from django.contrib import admin

from services.models import Service, ServiceItem, ServiceCategory


@admin.register(Service)
class ServiceAdmin(admin.ModelAdmin):

    list_display = ['id', 'created', 'customer', 'item', 'price']


@admin.register(ServiceItem)
class ServiceItemAdmin(admin.ModelAdmin):

    search_fields = ['name', 'code']
    list_display = ['id', 'name', 'code', 'price']


@admin.register(ServiceCategory)
class ServiceCategoryAdmin(admin.ModelAdmin):

    search_fields = ['name']
    list_display = ['id', 'name']




from django.db import models

from model_search import model_search
from basement.utils import clean_code


class ServiceItemQuerySet(models.QuerySet):

    def search(
            self,
            code=None,
            category=None,
            query=None):

        queryset = self

        if category:
            queryset = queryset.filter(category=category)

        if code:
            print(clean_code(code))
            queryset = queryset.filter(code__iexact=clean_code(code))

        if query:
            queryset = model_search(query, queryset, ['name'])

        return queryset




from django import forms
from django.utils.translation import ugettext_lazy as _

from basement.utils import get_date_from_request

from cap.fields import DatePickerField

from services.models import ServiceCategory


class ReportForm(forms.Form):

    date_from = DatePickerField(label=_('Date from'))

    date_to = DatePickerField(label=_('Date to'))

    def __init__(self, request):

        super().__init__(
            data={
                'date_from': get_date_from_request(request, 'date_from'),
                'date_to': get_date_from_request(request, 'date_to')
            }
        )

        self.is_valid()


class SearchServiceForm(forms.Form):

    code = forms.CharField(required=False)

    query = forms.CharField(required=False)

    category = forms.ModelChoiceField(
        queryset=ServiceCategory.objects.all(),
        required=False,
        widget=forms.HiddenInput)

    def clean(self):

        cleaned_data = {}

        for k, v in self.cleaned_data.items():
            if v:
                cleaned_data[k] = v

        return cleaned_data




from django.db import models

from services.querysets import ServiceItemQuerySet


class ServiceItemManager(models.Manager):

    def get_queryset(self):
        return ServiceItemQuerySet(self.model, using=self._db)

    def search(self, **kwargs):
        return self.get_queryset().search(**kwargs)




from django.urls import path

from services import views

app_name = 'services'


urlpatterns = [

    path('print-services/<int:sale_id>/', views.print_services,
         name='print-services'),

    path('print-service-items/', views.print_service_items,
         name='print-service-items'),

    path('report/', views.get_report, name='report'),

    path('items/', views.get_service_items, name='items'),

    path('<int:sale_id>/add-service/', views.add_service, name='add-service'),

    path('<int:service_id>/remove/', views.remove_service,
         name='remove-service')

]




from setuptools import setup, find_packages


version = '0.2'
url = 'https://github.com/pmaigutyak/mp-services'

with open('requirements.txt') as f:
    requires = f.read().splitlines()

setup(
    name='django-mp-services',
    version=version,
    description='Django services app',
    author='Paul Maigutyak',
    author_email='pmaigutyak@gmail.com',
    url=url,
    download_url='{}/archive/{}.tar.gz'.format(url, version),
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    install_requires=requires
)




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ServicesConfig(AppConfig):
    name = 'services'
    verbose_name = _("Services")


default_app_config = 'services.ServicesConfig'
